@@ -1,10 +1,10 @@
 # Array::PrintCols.pm
 #
-#    Copyright (C) 1995  Alan K. Stebbens <aks@hub.ucsb.edu>
+#    Copyright (C) 1996-2013  Alan K. Stebbens <aks@stebbens.org>
 #
-#    This program is free software; you can redistribute it and/or modify
+#    This program is free software: you can redistribute it and/or modify
 #    it under the terms of the GNU General Public License as published by
-#    the Free Software Foundation; either version 2 of the License, or
+#    the Free Software Foundation, either version 3 of the License, or
 #    (at your option) any later version.
 #
 #    This program is distributed in the hope that it will be useful,
@@ -13,22 +13,25 @@
 #    GNU General Public License for more details.
 #
 #    You should have received a copy of the GNU General Public License
-#    along with this program; if not, write to the Free Software
-#    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+#    along with this program.  If not, see <http://www.gnu.org/licenses/>.
 #
-# Author: Alan K. Stebbens <aks@hub.ucsb.edu>
+# Author: Alan K. Stebbens <aks@stebbens.org>
 #
-# $Id: PrintCols.pm,v 1.2 1998/01/18 07:15:57 aks Exp $
 
 package Array::PrintCols;
 
+$VERSION = '2.4';
+
 require 5.001;
+
 use Exporter; 
 @ISA = (Exporter);
 @EXPORT = qw( print_cols format_cols );
 
-sub min;
-sub max;
+use Carp;
+
+sub min(@);
+sub max(@);
 
 $PreSorted = 1;			# if set, do not need to sort
 
@@ -36,28 +39,35 @@ $PreSorted = 1;			# if set, do not need to sort
 #
 # $Array::PrintCols::PreSorted = 0;
 #
-
+#
 # format_cols \@array [, $col_width [, $total_width [, $indent ] ] ]
 #
 # or
 #
 # format_cols \@array [, -$columns [, $total_width, [, $indent ] ] ]
 #
+# or
+#
+# format_cols \%hash [, -$columns [, $total_width, [, $indent ] ] ]
+#
 #
 # This used to be "print_cols", which always sent everything to STDOUT.
 # Now, the work is done in "format_cols", which returns a string result.
 # Thanks to Gisle Aas <aas@bergen.sn.no> for the suggestion)
-
+#
 # Routine to format an array of values in alphabetically vertically 
 # sorted columns. 
 
-sub format_cols {
+sub format_cols($;@) {
     my($array)       = shift;
+    $array = $$array if ref($array) eq 'REF';
+    $array = [sort(keys(%$array))] if ref($array) eq 'HASH';
+    ref($array) eq 'ARRAY' or croak "arg 1 must be an ARRAY, HASH, or ARRAYREF\n";
     my($col_width)   = shift || 0;
     my($total_width) = shift || $ENV{'COLUMNS'} || 80;
     my($indent)      = shift || 0;
 
-    my($key,$max_len,$cols,$fmt,$cols,$col,$rows,$row);
+    my($key,$max_len,$cols,$fmt,$col,$rows,$row);
 
     my $o = '';		# start of the output
 
@@ -96,16 +106,19 @@ sub format_cols {
 
 # print_cols -- just print the results of format_cols.
 
-sub print_cols { print format_cols(@_); }
+sub print_cols($;@) { 
+   my $aref = shift;
+   print format_cols($aref,@_);
+}
 
-sub min { 
+sub min(@) { 
     my($min) = shift;
     local($_);
     foreach (@_) { $min = $_ if $min > $_; }
     $min;
 }
 
-sub max {
+sub max(@) {
     my($max) = shift;
     local($_);
     foreach (@_) { $max = $_ if $max < $_; }
@@ -132,45 +145,56 @@ C<print_cols \@I<array>, $I<colspec>, $I<total_width>;>
 
 C<print_cols \@I<array>, $I<colspec>, $I<total_width>, $I<indent>;>
 
+	     \@I<array> may also be an ARRAYREF, HASH, or HASHREF.
+
 $I<output> = C<format_cols> <same arguments as C<print_cols>>;
 
+
 C<$Array::PrintCols::PreSorted = 0;>
 
 =head1 DESCRIPTION
 
-This module exports two subroutine names: C<print_cols> and C<format_cols>.
+This module exports two subroutine names: C<print_cols> and
+C<format_cols>.
 
-The C<print_cols> subroutine prints the items of C<@I<array>> in multiple,
-alphabetically sorted vertical columns.  One, two, or three optional arguments
-may be given to C<print_cols> to control the width and number of the columns,
-the total width of the output, and indentation.  Reasonable defaults apply 
-in the absence of the optional arguments (or when given as the empty string 
-or zero).  Generally, the minimum width column is used when possible.
+The C<print_cols> subroutine prints the items of C<@I<array>> in
+multiple, alphabetically & vertically sorted columns.  In the case
+of the first argument being a HASH or HASHREF, only the keys of
+the hash are considered.
 
-If C<$I<colspec>> is given as a non-negative number, it is treated as
-the minimum width of the column; the actual width will be the maximum of
-this value and the lengths of all array items.
+One, two, or three optional arguments may be given to
+C<print_cols> to control the width and number of the columns, the
+total width of the output, and indentation.  Reasonable defaults
+apply in the absence of the optional arguments (or when given as
+the empty string or zero).  Generally, the minimum width column is
+used when possible.
 
-If C<$I<colspec>> is given as a negative number, its absolute value
-value is used to determine the total number of columns.  However, it cannot
-exceed the total number of columns possible based on the maximum length
-of all the array elements.
+If C<$I<colspec>> is given as a non-negative number, it is treated
+as the minimum width of the column; the actual width will be the
+maximum of this value and the lengths of all array items.
 
-If a third argument is supplied, it is used as the total width of the
-output.  The default for this value is the value of the environment
-variable C<$ENV{'COLUMNS'}>, if defined, or 80.
+If C<$I<colspec>> is given as a negative number, its absolute
+value value is used to determine the total number of columns.
+However, it cannot exceed the total number of columns possible
+based on the maximum length of all the array elements.
 
-If the fourth argument is given, it is used as the indent for all lines
-printed, which subtracts from the total width of the output.  This
-value defaults to zero (ie: no indention of the output).
+If a third argument is supplied, it is used as the total width of
+the output.  The default for this value is the value of the
+environment variable C<$ENV{'COLUMNS'}>, if defined, or 80.
 
-The variable C<$Array::PrintCols::PreSorted> controls whether or not the
-C<print_cols> subroutine will expect its array argument to already 
-be sorted.  If this variable is nil or zero, C<print_cols> will
-cause print out a sorted I<copy> of the input array.
+If the fourth argument is given, it is used as the indent for all
+lines printed, which subtracts from the total width of the output.
+This value defaults to zero (ie: no indention of the output).
 
-The routine C<format_cols> is called exactly like C<print_cols> but returns
-the formatted result as a string, rather than printing it.
+The variable C<$Array::PrintCols::PreSorted> controls whether or
+not the C<print_cols> subroutine will expect its array argument to
+already be sorted.  If this variable is nil or zero, C<print_cols>
+will cause print out a sorted I<copy> of the input array.
+
+The routine C<format_cols> is called exactly like C<print_cols>
+but returns the formatted result as a string, rather than printing
+it.  C<print_cols> actually invokes C<format_cols>, which does 
+the real work.
 
 =head1 EXAMPLES
 
@@ -181,17 +205,16 @@ the formatted result as a string, rather than printing it.
     # print in three columns with an indention of 1.
     print_cols \@commands, -3, 0, 1;
 
-
     # print the formatted result data to the result file
-    print RESULTFILE format_cols \@result_data;
+    print RESULTFILE format_cols(\@result_data);
 
 =head1 AUTHOR
 
-Copyright (C) 1995-1998  Alan K. Stebbens <aks@sgi.com>
+Copyright (C) 1996-2013  Alan K. Stebbens <aks@stebbens.org>
 
-This program is free software; you can redistribute it and/or modify
+This program is free software: you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
-the Free Software Foundation; either version 2 of the License, or
+the Free Software Foundation, either version 3 of the License, or
 (at your option) any later version.
 
 This program is distributed in the hope that it will be useful,
@@ -200,10 +223,9 @@ MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.
 
 You should have received a copy of the GNU General Public License
-along with this program; if not, write to the Free Software
-Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+along with this program.  If not, see <http://www.gnu.org/licenses/>.
 
 =head1 BUGS
 
-With Perl, you cannot default intervening arguments by leaving them empty;
-you must supply a zero or empty-string ('').
+Perl will not default intervening empty arguments; instead, use zero or the
+empty-string ('').
